' /////////////////////////////////////////////////////////////////////////////
{ 
 Propeller C3 Datalogger Demo
}
 

CON

'//////////////////////////////////////////////////////////////////////////////
' PROP CHIP / SYSTEM DEFINES //////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

  ' These settings are for 5 MHZ XTALS
  _clkmode = xtal1 + pll16x         ' enable external clock and pll times 16
  _xinfreq = 5_000_000              ' set frequency to 5 MHZ

  CLOCKS_PER_MICROSECOND = 5*16     ' simple xin*pll / 1_000_000                     
  CLOCKS_PER_MILLISECOND = 5000*16  ' simple xin*pll / 1_000
  
  _stack   = 128                    ' accomodate display memory and stack

'//////////////////////////////////////////////////////////////////////////////
' C3 PIN MAP DEFINES //////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

  ' status/debug pin also controls VGA buffer enable
  STATUS_LED_BUS_MUX     = 15 ' pin that controls VGA enable as well as a status/debug LED

  ' SPI pins
  SPI_SEL_CLK            = 8  ' used to clock the SPI counter that feeds 4-bit value to SPI selection decoder, clocks on rising edge
  SPI_SEL_CLR            = 25 ' used to clear the SPI counter back to 0000b (NULL SPI device), active LOW

  SPI_MOSI               = 9  ' SPI master out serial in to slave
  SPI_MISO               = 10 ' SPI master in serial out from slave, 10 for good C3, change to 7 for BROKEN C3 with YELLOW jumper.
  SPI_SCK                = 11 ' SPI clock from master to all slaves

  ' SPI channels defines 
  SPI_CHAN_NULL          = 0 ' NULL channel, disables all on/off board devices.
  SPI_CHAN_SRAM0         = 1 ' 32K SRAM Bank 0.
  SPI_CHAN_SRAM1         = 2 ' 32K SRAM Bank 1.
  SPI_CHAN_FLASH         = 3 ' 1MB FLASH Memory.
  SPI_CHAN_AD            = 4 ' MCP3202 2-Channel 12-bit A/D.
  SPI_CHAN_SD            = 5 ' Micro SD Card.

  ' interface header SPI selects
  SPI_CHAN_SPI6          = 6 ' Header Interface SPI6.
  SPI_CHAN_SPI7          = 7 ' Header Interface SPI7.

  ' PS/2 port pin defines
  PS2_DATA               = 26 ' PS/2 port clock pin
  PS2_CLK                = 27 ' PS/2 data pin               

  ' Port A pin defines
  PORTA0                 = 0
  PORTA1                 = 1
  PORTA2                 = 2
  PORTA3                 = 3
  PORTA4                 = 4
  PORTA5                 = 5
  PORTA6                 = 6
  PORTA7                 = 7

  ' Servo port 3-pin header output pins (shared with Port A P4..P7)
  ' note 1: each I/O pins is buffered thru a 50 ohm resistor to minimize long transmission line effects from cables
  ' note 2: each "servo" port has a 3 pin header [signal, power, ground], each power is feed from a jumper adjacent to the
  ' servo port and has a 22uF cap across it as a current source, thus the "servo" ports aside from using the servo 3-pin cable
  ' pin outs, CAN drive servos!!! Care must be taken not to sink too much current, if you find that the board resets under harsh
  ' conditions while driving multiple simulataneous servos then you can add a bulk 2200uF cap for energy storage. There are two holes
  ' pre-drilled by the 2.1mm power entry port, simply solder a 2200uF cap under or above the board, just make sure to get the polarity
  ' correct: minus to the left (next to left edge of board), plus to the right (next to switch).
  SERVO_P4               = 4 
  SERVO_P5               = 5
  SERVO_P6               = 6
  SERVO_P7               = 7


  ' Port B pin defines (muxed with VGA port)
  PORTB0                 = 16  
  PORTB1                 = 17
  PORTB2                 = 18
  PORTB3                 = 19
  PORTB4                 = 20
  PORTB5                 = 21
  PORTB6                 = 22
  PORTB7                 = 23

  ' VGA port pin defines
  VGA_HSYNC              = 16
  VGA_VSYNC              = 17
  VGA_B0                 = 18
  VGA_B1                 = 19
  VGA_G0                 = 20
  VGA_G1                 = 21
  VGA_R0                 = 22
  VGA_R1                 = 23

  ' NTSC/PAL composite video pin defines
  COMPVID0               = 12
  COMPVID1               = 13
  COMPVID2               = 14

  ' Audio output pin define (PWM)
  AUDIO_MONO             = 24

  ' I2C pin defines (standard Prop pins)
  I2C_SCL                = 28
  I2C_SDA                = 29
 
  ' TX/RX pin defines (standard Prop pins)
  UART_TX                = 31
  UART_RX                = 30

   ' ASCII codes for ease of parser development
  ASCII_A       = 65
  ASCII_B       = 66
  ASCII_C       = 67
  ASCII_D       = 68
  ASCII_E       = 69
  ASCII_F       = 70
  ASCII_G       = 71
  ASCII_H       = 72
  ASCII_O       = 79  
  ASCII_P       = 80
  ASCII_Z       = 90
  ASCII_0       = 48
  ASCII_9       = 57
  ASCII_LEFT   = $C0
  ASCII_RIGHT  = $C1
  ASCII_UP     = $C2
  ASCII_DOWN   = $C3 
  ASCII_BS     = $08 ' backspace
  ASCII_DEL    = $7F ' delete   
  ASCII_LF     = $0A ' line feed 
  ASCII_CR     = $0D ' carriage return
  ASCII_ESC    = $1B ' escape
  ASCII_HEX    = $24 ' $ for hex
  ASCII_BIN    = $25 ' % for binary
  ASCII_LB     = $5B ' [ 
  ASCII_SEMI   = $3A ' ; 
  ASCII_EQUALS = $3D ' = 
  ASCII_PERIOD = $2E ' .
  ASCII_COMMA  = $2C ' ,
  ASCII_SHARP  = $23 ' #
  ASCII_NULL   = $00 ' null character
  ASCII_SPACE  = $20 ' space
  ASCII_TAB    = $09 ' horizontal tab

  ' null pointer, null character
  NULL         = 0

  ' SRAM function helper constants
  SPI_MODE_BYTE       = %00_00000_1
  SPI_MODE_PAGE       = %10_00000_1
  SPI_MODE_SEQUENTIAL = %01_00000_1


  DO = 10
  CLK =11
  DI = 9
  CS = 3

' SPI (serial peripheral interface) interface pins
  spiDO  = 10
  spiCLK = 11
  spiDI  = 9       
   
'//////////////////////////////////////////////////////////////////////////////
' VARS SECTION ////////////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////        
  
VAR

  ' generic globals
  long g_data, g_data2, g_status    
  long g_data32, g_data16, g_spi_result
  long g_key, g_temp1, g_temp2, g_temp3, g_index, g_length
  long g_tempx, g_tempy
  long fTemp, fResult  

  byte g_sbuffer[512], myBuff[20]

'//////////////////////////////////////////////////////////////////////////////
'OBJS SECTION /////////////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

OBJ
 
 c3         : "C3_Lib.spin"
 sd         : "fsrw.spin"
 vga        : "VGA_Text_010.spin"
 kbd        : "keyboard_010.spin"
 math       : "FloatMath.spin"
 mathStr    : "FloatString.spin"

 
'//////////////////////////////////////////////////////////////////////////////
' MAIN ENTRY POINT TO DEMO/////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

PUB Main : status  | mount

  ' let the system initialize 
  waitcnt(clkfreq + cnt)

  kbd.start(PS2_DATA, PS2_CLK)  
  kbd.clearkeys
        
  vga.start(%10111)
  
  ' delay a moment
  waitcnt(clkfreq/100 + cnt)
  
  ' /////////////////////////////////////////////////////////////////////////////
  ' INITIALIZE I/O PINS /////////////////////////////////////////////////////////
  ' /////////////////////////////////////////////////////////////////////////////

  ' blink the status LED 3 times to show board is "alive", then enable VGA (leave LOW)
  DIRA[STATUS_LED_BUS_MUX] := 1 ' set to output
  OUTA[STATUS_LED_BUS_MUX] := 0

  repeat 6
    OUTA[STATUS_LED_BUS_MUX] := !OUTA[STATUS_LED_BUS_MUX]
    repeat 25_000 

  ' turn on VGA buffer
  OUTA[STATUS_LED_BUS_MUX] := 0       ' 0=on, 1=off

     
  ' enter test loop
      
         ' initialize C3 SPI IO since the A/D is on the SPI
        c3.SPI_Init
         ' initialize the SRAMs into sequential access mode    
        c3.SRAM_Init( SPI_MODE_SEQUENTIAL )     
         ' initialize AD converter
        c3.AD_Init
         ' initialize Flash RAM
        c3.Flash_Open
        c3.Flash_Erase_Chip(1)
        
        c3.SPI_Select_Channel( SPI_CHAN_SD )        
         ' initialize SD buffers and get FAT data 
        mount := \sd.mount_explicit(DO,CLK,DI,CS)
        if mount < 0
           vga.str(string("Failed to mount", 13))
          repeat
      sd.release
      c3.SPI_Select_Channel( SPI_CHAN_NULL )
              
      repeat             
         
        ' clear screen   
        vga.Out( $00 )
        ' call the test
        c3_Datalog
       
' end Main

' /////////////////////////////////////////////////////////////////////////////
' /////////////////////////////////////////////////////////////////////////////

PUB c3_Datalog   | char
  
  '/////////////////////////////////////////////////////////////////////////////
  ' C3 DAQ-Datalog Demo   //////////////////////////////////////////////////////
  '/////////////////////////////////////////////////////////////////////////////
 
  vga.str( string ("C3 Datalogger Demo ", 13) )  
  vga.str( string ("Connect an LM34 probe to A/D", 13) )    
  vga.str( string ("port analog input channel 1.", 13) )  
  vga.str( string ("The port pinout is:", 13) )   
  vga.str( string ("      |--------------|",13) )   
  vga.str( string ("Pin 4 |  Analog In 0 |",13) )   
  vga.str( string ("      |--------------|",13) )    
  vga.str( string ("Pin 3 |  Analog In 1 |",13) )   
  vga.str( string ("      |--------------|",13) )     
  vga.str( string ("Pin 2 |  +5V Out     |",13) )    
  vga.str( string ("      |--------------|",13) )   
  vga.str( string ("Pin 1 |     GND      |",13) )   
  vga.str( string ("      |--------------|",13) ) 
  vga.str( string ("Press any key...") )

  ' wait for keypress
   kbd.getkey
  ' flush buffer  
   kbd.clearkeys
   

   
   vga.Out( $00 )

   vga.sety(5)
   vga.setx(10)
   vga.str( string ("Temperature"))
   vga.out(13)
   
   c3.SPI_Init 
     
      ' display A/D channel until user hits a key
      repeat while (kbd.gotkey == FALSE) 
        ' read channel 1, SPI transaction is 17 bits total, control bits indicate single ended mode, channel 1
        g_data := c3.AD_Read( 1 )
        fTemp := math.FDiv(math.FFloat(g_data), float(4096))
        fTemp := math.FMul(fTemp, float(5))
        fTemp := math.FMul(fTemp, float(100))
        g_temp3 := math.FSub(fTemp, float(100))
        if  g_temp3 > 0
         mathStr.SetPrecision(4)
        else
         mathStr.SetPrecision(3)
        vga.sety(7)
        vga.setx(13)    
        vga.str( mathStr.FloatToString(fTemp))
        vga.out(" ") 
        waitcnt(clkfreq/10 + cnt)
       
   vga.Out( $00 )
   
   vga.str(string("Log file name:"))
   getline(@myBuff, 20)   
   c3.SPI_Select_Channel( SPI_CHAN_SD )
   
   if (sd.popen(@myBuff, "w"  )<> 0 )
    vga.str(string(13, 13, "File error."))
    repeat
    
   sd.release
   
   c3.SPI_Select_Channel( SPI_CHAN_NULL )
   
   vga.Out( $00 )    
   vga.str(string("Storing temperatures"))
     
   g_length := 10
   
    
      ' loop g_length times and write memory each time 
      repeat g_temp1 from 0 to g_length
        c3.SPI_Init
        ' read channel 1
        g_data := c3.AD_Read(1)
        
        ' store raw data
        ' write data into SRAM Bank 1 (high nibble)
        g_sbuffer[0] := g_data >> 8  
        c3.SRAM_Write( 1, g_temp1, 1, @g_sbuffer[0])
       ' write data into SRAM Bank 0 (low byte)
        g_sbuffer[0] := g_data & $FF  
        c3.SRAM_Write( 0, g_temp1, 1, @g_sbuffer[0])
        
       ' convert to float and store in Flash RAM
        fTemp := math.FDiv(math.FFloat(g_data), float(4096))
        fTemp := math.FMul(fTemp, float(5))
        fTemp := math.FMul(fTemp, float(100))
        c3.Flash_Write( g_temp1*4, @fTemp, 4)

        c3.SPI_Select_Channel( SPI_CHAN_SD )
        sd.pputs(mathStr.FloatToString(math.FFloat(g_temp1)))
        sd.pputc($2C)
        sd.pputs(mathStr.FloatToString(fTemp))
        sd.pputs(string(" ",13))       
        sd.release
        
        if (g_temp1 // 100 == 0)
         vga.out(".")
   sd.pclose    
   vga.Out( $00 )    

      c3.SPI_Init
      ' loop g_length times and read SRAM 
      repeat g_temp1 from 0 to g_length
       ' read raw data
       ' read data from SRAM Bank 1 (high nibble)
        c3.SRAM_Read( 1, g_temp1, 1, @g_sbuffer[1])
       ' read data from SRAM Bank 0 (low byte)
        c3.SRAM_Read( 0, g_temp1, 1, @g_sbuffer[0])
           
        g_data := g_sbuffer[1]
        g_data := g_data << 8 + g_sbuffer[0] 
        
        fTemp := math.FDiv(math.FFloat(g_data), float(4096))
        fTemp := math.FMul(fTemp, float(5))
        fTemp := math.FMul(fTemp, float(100))
        vga.sety(5)
        vga.setx(11)
        vga.str( string ("SRAM["))    
        vga.dec( g_temp1 )  
        vga.str( string ("]"))
        vga.sety(7)
        vga.setx(13)
        vga.str(mathStr.FloatToString(fTemp))

   vga.out( $00 )       

      ' loop g_length times and read Flash
      repeat g_temp1 from 0 to g_length
        vga.sety(5)
        vga.setx(10)
        vga.str( string ("Flash["))    
        vga.dec( g_temp1*4 )  
        vga.str( string ("]"))
        vga.sety(7)
        vga.setx(12)
        c3.Flash_Read(g_temp1*4, @fTemp, 4)
        vga.str(mathStr.FloatToString(fTemp))

     
   c3.Flash_Close

   vga.out( $00 )
   
   c3.SPI_Init
   c3.SPI_Select_Channel(SPI_CHAN_SD)
   sd.popen(@myBuff, "r" ) 
      'read SD     
      repeat sd.get_filesize    
        vga.out(sd.pgetc)     

   sd.pclose
   
   
   ' stop program and wait for restart
 
   kbd.clearkeys   
   vga.str( string (13,"Press any key...") )
   kbd.getkey
  
' end AD_Test




PUB getline(string_ptr, maxlength) | cx, cy, ch, sindex
' DESCRIPTION:
'
' INPUTS:
' OUTPUTS:
' -----------------------------------------------------------------------------

  ' initialize single line editor
  sindex := 0
  cx := vga.getx
  cy := vga.gety

  repeat
    ' draw cursor
    vga.out( "_" )
    vga.setx( vga.getx - 1 )
  
    ' get next key from keyboard
    ch := kbd.getkey

    ' check for printable characters
    if ( (ch => $20) and (ch =< $7E) )
      ' check if we are at end of line
      if (sindex < maxlength-1)

        byte[string_ptr][sindex++] := ch
     
        ' erase the character and cursor with special call to draw a blank (added to terminal driver)
        vga.erase

        ' print the character 
        vga.out( ch )
        
    elseif (ch == $C8) ' backspace character
      if (sindex > 0)
        ' insert space
        byte[string_ptr][sindex--] := $20

        ' erase the character and cursor
        vga.erase
        vga.setx( vga.getx - 1 )
        vga.erase

    elseif (ch == $0D) ' enter character
      ' null terminate
      byte[string_ptr][sindex] := 0
      ' return length
      return(sindex)      
  
' end getline